#include "Speech Engine.h"


// This is the global object for the speech engine
SPEECH_ENGINE Speech;



//////////////////////////////////////////////////////////////////////////////////////////////////
// Speech Engine class definitions																//
//	This section defines all of the methods (member functions) of the speech engine class.		//
//////////////////////////////////////////////////////////////////////////////////////////////////

// Class:	SPEECH_ENGINE
// Method:	Speech engine default constructor
//
// Purpose:  Initializes COM and the voice interface
SPEECH_ENGINE::SPEECH_ENGINE ()
{
	// Clear the object of all data
	ZeroMemory ( this, sizeof ( *this ) );
	

	// Initialize COM; Parameter is reserved, and must be NULL
	CoInitialize ( NULL );


	// Create the voice instance
	CoCreateInstance 
		( 
			CLSID_SpVoice,			// The object's class identifier; speech voice
			NULL,					// Used for COM aggregation; Not used in this program
			CLSCTX_ALL,				// Context of the object; object uses all contexts
			IID_ISpVoice,			// Object identifier for the interface; speech voice
			(void**)&Voice			// The resulting interface; a voice interface
		);
}



// Class:	SPEECH_ENGINE
// Method:	Speech engine destructor
//
// Purpose:  Releases the voice interface (if needed), and shuts down COM
SPEECH_ENGINE::~SPEECH_ENGINE ()
{
	// Safely release the voice interface
	if ( Voice != NULL ) Voice -> Release ();
	Voice = NULL;

	// Uninitialize COM
	CoUninitialize ();
}



// Class:	SPEECH_ENGINE
// Method:  Speak ()
//
// Purpose:  This function takes the current contents of the text buffer, and
//	speaks, using the object's voice.  The buffer is then cleared for reuse.
void SPEECH_ENGINE::Speak ()
{
	// Wide character buffer
	WCHAR WideBuffer [ MaxTextLength ];

	// Convert to wide characters; Convert from ASCII characters to wide characters
	MultiByteToWideChar ( CP_ACP, 0, TextBuffer, -1, WideBuffer, MaxTextLength );


	// Speak the contents of the buffer
	Voice -> Speak 
		( 
			WideBuffer,		// Text buffer, in wide characters
			SPF_IS_XML,		// Parse for XML tags
			NULL			// Current input stream number; we don't use; we don't care
		);


	// Clear the text buffer
	ZeroMemory ( TextBuffer, sizeof ( TextBuffer ) );
}



// Class:	SPEECH_ENGINE
// Method:  SpeakLastError ()
//
// Purpose:  This function retrieves the last error message that the system recieved.
//	It then formats it in the text buffer, and calls the Speak method, thereby rendering
//	the last error message.
void SPEECH_ENGINE::SpeakLastError ()
{
	// Retrieve the error text string, and store it in the text buffer
	FormatMessage 
		( 
			FORMAT_MESSAGE_FROM_SYSTEM,		// Format a system error message
			NULL,							// Message source; ignored for system messages
			GetLastError (),				// Message code; the last error
			0,								// Language identifier for the message; use default
			TextBuffer,						// The resulting string buffer
			MaxTextLength,					// The maximum size of the buffer
			NULL							// Optional parameters; not needed here
		);


	// Speak the contents of the buffer
	Speak ();
}



//////////////////////////////////////////////////////////////////////////////////////////////////
// Overloaded Operator Definitions																//
//																								//
//	The following section contains the definitions for the overloaded operators.  The			//
// definitions are very similar, most of the time, and I'm pretty sure I can generalize the		//
// operators, if I use templates.  Right now, I'm not too comfortable with templates,			//
// unfortunately.																				//
//////////////////////////////////////////////////////////////////////////////////////////////////

SPEECH_ENGINE& operator << ( SPEECH_ENGINE& Engine, char* Text )
{
	// Append the new text to the buffer
	strcat ( Engine.TextBuffer, Text );

	// Flush the buffer
	Engine.Speak ();


	return Engine;
}



SPEECH_ENGINE& operator << ( SPEECH_ENGINE& Engine, int Int )
{
	// Temporary string buffer
	char TempString [100];

	// Convert the integer to a temporary string
	sprintf ( TempString, "%i", Int );

	// Append the temporary string to the buffer
	strcat ( Engine.TextBuffer, TempString );

	// Flush the buffer
	Engine.Speak ();


	return Engine;
}



SPEECH_ENGINE& operator << ( SPEECH_ENGINE& Engine, short Short )
{
	// Temporary string buffer
	char TempString [100];

	// Convert the integer to a temporary string
	sprintf ( TempString, "%i", Short );

	// Append the temporary string to the buffer
	strcat ( Engine.TextBuffer, TempString );

	// Flush the buffer
	Engine.Speak ();


	return Engine;
}



SPEECH_ENGINE& operator << ( SPEECH_ENGINE& Engine, long Long )
{
	// Temporary string buffer
	char TempString [100];

	// Convert the integer to a temporary string
	sprintf ( TempString, "%i", Long );

	// Append the temporary string to the buffer
	strcat ( Engine.TextBuffer, TempString );

	// Flush the buffer
	Engine.Speak ();


	return Engine;
}



SPEECH_ENGINE& operator << ( SPEECH_ENGINE& Engine, float Float )
{
	// Temporary string buffer
	char TempString [100];

	// Convert the float to a temporary string
	sprintf ( TempString, "%6.6f", Float );

	// Append the temporary string to the buffer
	strcat ( Engine.TextBuffer, TempString );

	// Flush the buffer
	Engine.Speak ();


	return Engine;
}



SPEECH_ENGINE& operator << ( SPEECH_ENGINE& Engine, double Double )
{
	// Temporary string buffer
	char TempString [100];

	// Convert the double to a temporary string
	sprintf ( TempString, "%10.10d", Double );

	// Append the temporary string to the buffer
	strcat ( Engine.TextBuffer, TempString );

	// Flush the buffer
	Engine.Speak ();


	return Engine;
}



SPEECH_ENGINE& operator << ( SPEECH_ENGINE& Engine, long double Double )
{
	// Temporary string buffer
	char TempString [100];

	// Convert the double to a temporary string
	sprintf ( TempString, "%10.10d", Double );

	// Append the temporary string to the buffer
	strcat ( Engine.TextBuffer, TempString );

	// Flush the buffer
	Engine.Speak ();


	return Engine;
}



SPEECH_ENGINE& operator << ( SPEECH_ENGINE& Engine, unsigned char Byte )
{
	// Append the byte to the buffer
	strcat ( Engine.TextBuffer, (char*)&Byte );

	// Speak the contents of the buffer
	Engine.Speak ();


	return Engine;
}



SPEECH_ENGINE& operator << ( SPEECH_ENGINE& Engine, unsigned short Short )
{
	// Temporary string buffer
	char TempString [100];

	// Convert the integer to a temporary string
	sprintf ( TempString, "%i", Short );

	// Append the temporary string to the buffer
	strcat ( Engine.TextBuffer, TempString );

	// Flush the buffer
	Engine.Speak ();

	
	return Engine;
}



SPEECH_ENGINE& operator << ( SPEECH_ENGINE& Engine, unsigned long Long )
{
	// Temporary string buffer
	char TempString [100];

	// Convert the integer to a temporary string
	sprintf ( TempString, "%i", Long );

	// Append the temporary string to the buffer
	strcat ( Engine.TextBuffer, TempString );

	// Flush the buffer
	Engine.Speak ();

	
	return Engine;
}



